(*
    Access to the Windows 95 tray notification area for
    16-bit applications
    Copyright (c) 1995-2003 Alessandro Scotti
    http://www.walkofmind.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Tray95;
interface
uses WinTypes, WinProcs;

const
  (* Flags *)
  NIF_MESSAGE = 1;
  NIF_ICON    = 2;
  NIF_TIP     = 4;
  (* Messages *)
  NIM_ADD     = 0;
  NIM_MODIFY  = 1;
  NIM_DELETE  = 2;
type
  (* Notify icon data, 16 bit version (tweaked for Borland Pascal) *)
  PNotifyIconData16 = ^TNotifyIconData16;
  TNotifyIconData16 = record
    hWindow       : HWnd;
    uId           : word;
    uFlags        : word;
    uCallback     : word;
    hIcon         : THandle;
    sTip          : string[63];
  end;

function Shell_NotifyIcon16(wMsg: word; pNID: PNotifyIconData16 ): word;

implementation

const
  WM_COPYDATA = $4A;
const
  hTrayWindow : HWnd = 0;
  wWinVersion : word = 0;
type
  (* Structure for WM_COPYDATA message *)
  TCopyData = record
    dwData: longint;
    cbData: longint;
    lpData: pointer;
  end;
  (* Structure for shell icon notification *)
  TNotifyIconData32 = record
    lTag          : longint;  (* Tag *)
    dwMsg         : longint;  (* Message *)
    cbSize        : longint;
    hWnd          : longint;
    uID           : longint;
    uFlags        : longint;
    uCallback     : longint;
    hIcon         : longint;
    szTip         : array[ 0..63 ] of char;
  end;

function Shell_NotifyIcon16(wMsg: word; pNID: PNotifyIconData16 ): word;
var
  stCD    : TCopyData;
  stNID32 : TNotifyIconData32;
  iTmp    : integer;
begin
  Shell_NotifyIcon16 := 0;
  (* Get Windows version if needed *)
  if( wWinVersion = 0 ) then begin
    wWinVersion := GetVersion;
    wWinVersion := Swap( wWinVersion );
  end;
  (* Check Windows version (try if 3.51 or greater) *)
  if( wWinVersion < $0333 ) then
    Exit;
  (* Search shell tray window if needed *)
  if( hTrayWindow = 0 ) then begin
    FindWindow( 'Shell_TrayWnd', nil );
    if( hTrayWindow = 0 ) then
      Exit;
  end;
  (* Fill in the proper structures *)
  with stNID32 do begin
    lTag := $34753423;
    dwMsg := wMsg;
    cbSize := sizeof(TNotifyIconData32) - 8;
    hWnd := longint( pNID^.hWindow );
    uID := pNID^.uID;
    uFlags := pNID^.uFlags and $0007;
    uCallback := pNID^.uCallback;
    hIcon := longint(pNID^.hIcon);
    for iTmp := 1 to Length(pNID^.sTip) do
      szTip[iTmp-1] := pNID^.sTip[iTmp];
    szTip[ Length(pNID^.sTip) ] := #0;
  end;
  with stCD do begin
    dwData := $0001;
    cbData := sizeof(TNotifyIconData32);
    lpData := @stNID32;
  end;
  (* Send message and exit *)
  Shell_NotifyIcon16 := SendMessage( hTrayWindow,
    WM_COPYDATA,
    word( pNID^.hWindow ),
    longint( @stCD ) );
end;

end.
